#!/usr/bin/env python3
"""Intel-specific GPU monitoring implementation."""

import re
import subprocess
from pathlib import Path
from typing import List, Optional, Dict, Any

from .base import BaseGPUMonitor, GPUInfo


class IntelGPUMonitor(BaseGPUMonitor):
    """Intel GPU monitoring implementation."""
    
    def _find_smi(self) -> Optional[Path]:
        """Find Intel GPU monitoring tool."""
        common_paths = [
            '/usr/bin/intel_gpu_top',
            '/usr/local/bin/intel_gpu_top'
        ]
        for path in common_paths:
            if Path(path).exists():
                return Path(path)
        return None

    def _parse_gpu_info(self, output: str) -> List[Dict[str, Any]]:
        """Parse intel_gpu_top output for all GPUs."""
        if not output:
            return []

        gpus = []
        current_gpu = {}
        
        # Parse output for each GPU
        gpu_sections = output.split('Intel GPU device')
        for section in gpu_sections[1:]:  # Skip first empty split
            try:
                # Extract device info
                device_match = re.search(r'(\d+):\s*([^\n]+)', section)
                if device_match:
                    index = int(device_match.group(1))
                    name = device_match.group(2).strip()
                    
                    # Extract utilization
                    util_match = re.search(r'Render/3D/0\s+([0-9.]+)%', section)
                    utilization = float(util_match.group(1)) if util_match else 0.0
                    
                    # Extract memory info (if available)
                    mem_match = re.search(r'Memory: (\d+)MB used / (\d+)MB total', section)
                    if mem_match:
                        memory_used = float(mem_match.group(1))
                        memory_total = float(mem_match.group(2))
                    else:
                        memory_used = 0.0
                        memory_total = 0.0
                    
                    # Extract temperature (if available)
                    temp_match = re.search(r'GPU temperature: (\d+)°C', section)
                    temperature = float(temp_match.group(1)) if temp_match else 0.0
                    
                    # Extract power (if available)
                    power_match = re.search(r'Power: (\d+\.\d+)W', section)
                    power = float(power_match.group(1)) if power_match else 0.0
                    
                    # Extract processes
                    processes = []
                    proc_section = re.search(r'Processes:\n(.*?)(?=\n\n|\Z)', section, re.DOTALL)
                    if proc_section:
                        proc_lines = proc_section.group(1).strip().split('\n')
                        for line in proc_lines:
                            if line.strip():
                                parts = line.split()
                                if len(parts) >= 2:
                                    try:
                                        pid = int(parts[0])
                                        processes.append({
                                            'pid': pid,
                                            'name': ' '.join(parts[1:]),
                                            'memory': 0.0  # Memory per process not available
                                        })
                                    except (ValueError, IndexError):
                                        continue
                    
                    gpus.append({
                        'index': index,
                        'name': name,
                        'utilization': utilization,
                        'memory_used': memory_used,
                        'memory_total': memory_total,
                        'temperature': temperature,
                        'power_draw': power,
                        'processes': processes
                    })
                    
            except (ValueError, AttributeError, IndexError):
                continue
                
        return gpus

    def get_gpu_info(self) -> List[GPUInfo]:
        """Get Intel GPU information for all devices."""
        if not self.smi_path:
            return []

        try:
            # Get initial device list
            cmd = [str(self.smi_path), '-L']
            result = self._run_smi_command(cmd)
            if not result:
                return []
                
            # Get detailed info
            cmd = [str(self.smi_path), '-J']  # JSON output if supported
            result = self._run_smi_command(cmd)
            if not result:
                return []
                
            gpu_info = self._parse_gpu_info(result)
            
            return [GPUInfo(
                index=info['index'],
                name=info['name'],
                utilization=info['utilization'],
                memory_used=info['memory_used'],
                memory_total=info['memory_total'],
                temperature=info['temperature'],
                power_draw=info['power_draw'],
                power_limit=0.0,  # Power limit not available
                processes=info['processes']
            ) for info in gpu_info]
            
        except Exception:
            return []

    def _run_smi_command(self, cmd: List[str]) -> Optional[str]:
        """Run an Intel GPU monitoring command."""
        try:
            result = subprocess.run(
                cmd,
                capture_output=True,
                text=True,
                check=True,
                timeout=5
            )
            return result.stdout
        except (subprocess.SubprocessError, FileNotFoundError, subprocess.TimeoutExpired):
            return None



#!/usr/bin/env python3
"""Tab navigation component for the UI."""

import curses
from typing import List, Optional

from ..display import Display


class TabsComponent:
    """Renders navigation tabs."""
    
    def __init__(self, display: Display):
        """Initialize the tabs component.
        
        Args:
            display: Display instance
        """
        self.display = display
        self.tabs = ['OVERVIEW', 'AI PROCESSES', 'GPU', 'MEMORY', 'CPU']
    
    def render(self, selected_tab: int, y: int,
               custom_tabs: Optional[List[str]] = None) -> int:
        """Render the navigation tabs.
        
        Args:
            selected_tab: Index of selected tab
            y: Starting Y coordinate
            custom_tabs: Optional custom tab labels
            
        Returns:
            Next Y coordinate
        """
        tabs = custom_tabs if custom_tabs is not None else self.tabs
        tab_width = self.display.width // len(tabs)
        
        for i, tab in enumerate(tabs):
            x = i * tab_width
            attr = (curses.color_pair(2) if i == selected_tab 
                   else curses.color_pair(1))
            
            # Center the tab text
            padded_tab = f" {tab} "
            tab_text = self.display.center_text(padded_tab, tab_width)
            
            self.display.safe_addstr(y, x, tab_text, attr)
        
        return y + 1
    
    def get_tab_count(self, custom_tabs: Optional[List[str]] = None) -> int:
        """Get the number of tabs.
        
        Args:
            custom_tabs: Optional custom tab labels
            
        Returns:
            Number of tabs
        """
        return len(custom_tabs if custom_tabs is not None else self.tabs)
    
    def handle_tab_input(self, key: int, current_tab: int) -> int:
        """Handle tab navigation input.
        
        Args:
            key: Input key code
            current_tab: Current tab index
            
        Returns:
            New tab index
        """
        if key == curses.KEY_RIGHT:
            return (current_tab + 1) % self.get_tab_count()
        elif key == curses.KEY_LEFT:
            return (current_tab - 1) % self.get_tab_count()
        return current_tab



"""AITop - A system monitor focused on AI/ML workload monitoring."""

from .core import (
    GPUMonitorFactory,
    AIProcessMonitor,
    SystemMemoryMonitor,
    GPUInfo,
    MemoryStats
)

__version__ = "0.1.0"
__all__ = [
    'GPUMonitorFactory',
    'AIProcessMonitor',
    'SystemMemoryMonitor',
    'GPUInfo',
    'MemoryStats',
]


#!/usr/bin/env python3
"""GPU information panel component."""

import curses
from typing import List, Optional

from ...core.gpu.base import GPUInfo
from ..display import Display


class GPUPanel:
    """Renders GPU information panel."""
    
    def __init__(self, display: Display):
        """Initialize the GPU panel.
        
        Args:
            display: Display instance
        """
        self.display = display
        
    def _render_gpu_header(self, gpu: GPUInfo, y: int, x: int, 
                          vendor: str) -> int:
        """Render GPU header with name and vendor.
        
        Args:
            gpu: GPU information
            y: Starting Y coordinate
            x: Starting X coordinate
            vendor: GPU vendor string
            
        Returns:
            Next Y coordinate
        """
        vendor_prefix = ""
        if all(v.upper() not in gpu.name.upper() 
               for v in ["NVIDIA", "AMD", "INTEL"]):
            vendor_prefix = f"{vendor.upper()} "
            
        header = f"GPU {gpu.index}: {vendor_prefix}{gpu.name}"
        self.display.safe_addstr(y, x, header, 
                               curses.color_pair(1) | curses.A_BOLD)
        return y + 2
    
    def _render_utilization_bar(self, gpu: GPUInfo, y: int, x: int, 
                              width: int) -> int:
        """Render GPU utilization bar.
        
        Args:
            gpu: GPU information
            y: Starting Y coordinate
            x: Starting X coordinate
            width: Maximum width for the bar
            
        Returns:
            Next Y coordinate
        """
        bar, color = self.display.create_bar(gpu.utilization, width)
        line = f"Util  [{bar}] {gpu.utilization:5.1f}%"
        self.display.safe_addstr(y, x, line, color)
        return y + 1
    
    def _render_memory_bar(self, gpu: GPUInfo, y: int, x: int, 
                          width: int) -> int:
        """Render memory usage bar.
        
        Args:
            gpu: GPU information
            y: Starting Y coordinate
            x: Starting X coordinate
            width: Maximum width for the bar
            
        Returns:
            Next Y coordinate
        """
        mem_percent = (gpu.memory_used / gpu.memory_total * 100 
                      if gpu.memory_total else 0)
        bar, color = self.display.create_bar(mem_percent, width)
        
        line = (f"Mem   [{bar}] {gpu.memory_used/1024:5.1f}GB / "
                f"{gpu.memory_total/1024:5.1f}GB")
        self.display.safe_addstr(y, x, line, color)
        return y + 1
    
    def _render_temperature(self, gpu: GPUInfo, y: int, x: int) -> int:
        """Render temperature information.
        
        Args:
            gpu: GPU information
            y: Starting Y coordinate
            x: Starting X coordinate
            
        Returns:
            Next Y coordinate
        """
        if gpu.temperature > 0:
            line = f"Temp  {gpu.temperature:5.1f}°C"
            self.display.safe_addstr(y, x, line, 
                                   self.display.get_color(gpu.temperature))
            return y + 1
        return y
    
    def _render_power(self, gpu: GPUInfo, y: int, x: int) -> int:
        """Render power usage information.
        
        Args:
            gpu: GPU information
            y: Starting Y coordinate
            x: Starting X coordinate
            
        Returns:
            Next Y coordinate
        """
        if gpu.power_draw > 0 and gpu.power_limit > 0:
            power_percent = (gpu.power_draw / gpu.power_limit * 100)
            line = f"Power {gpu.power_draw:5.1f}W / {gpu.power_limit:5.1f}W"
            self.display.safe_addstr(y, x, line, 
                                   self.display.get_color(power_percent))
            return y + 1
        return y
    
    def _render_processes(self, gpu: GPUInfo, y: int, x: int) -> int:
        """Render GPU process information.
        
        Args:
            gpu: GPU information
            y: Starting Y coordinate
            x: Starting X coordinate
            
        Returns:
            Next Y coordinate
        """
        if not gpu.processes:
            return y
            
        y += 1
        self.display.safe_addstr(y, x, "Running Processes:", 
                               curses.color_pair(5))
        y += 1
        
        for proc in gpu.processes:
            line = (f"  PID {proc['pid']:6d} | {proc['memory']:5.0f}MB | "
                   f"{proc['name']}")
            self.display.safe_addstr(y, x, line, curses.color_pair(6))
            y += 1
            
        return y + 1
    
    def render(self, gpu_info: List[tuple[GPUInfo, str]], start_y: int = 3, 
               indent: int = 2) -> int:
        """Render the complete GPU panel.
        
        Args:
            gpu_info: List of tuples containing (GPU information, vendor string)
            start_y: Starting Y coordinate
            indent: Left indentation
            
        Returns:
            Next Y coordinate
        """
        if not gpu_info:
            self.display.safe_addstr(start_y, indent, 
                                   "No compatible GPUs detected",
                                   curses.color_pair(3))
            return start_y + 1
            
        y = start_y
        bar_width = min(50, self.display.width - 35)
        
        for gpu, vendor in gpu_info:
            # Render GPU sections
            y = self._render_gpu_header(gpu, y, indent, vendor)
            y = self._render_utilization_bar(gpu, y, indent, bar_width)
            y = self._render_memory_bar(gpu, y, indent, bar_width)
            y = self._render_temperature(gpu, y, indent)
            y = self._render_power(gpu, y, indent)
            y = self._render_processes(gpu, y, indent)
            
        return y



"""Configuration module."""

import json
from pathlib import Path

def load_process_patterns():
    """Load AI process patterns from JSON configuration."""
    patterns_path = Path(__file__).parent / 'ai_process_patterns.json'
    with open(patterns_path) as f:
        return json.load(f)

__all__ = ['load_process_patterns']


#!/usr/bin/env python3
"""AITop - A system monitor focused on AI/ML workload monitoring."""

import curses
import signal
import sys
import threading
import time
from dataclasses import dataclass
from queue import Queue
from typing import Optional, List, Tuple, Any

from aitop.core.gpu.factory import GPUMonitorFactory
from aitop.core.process.monitor import AIProcessMonitor
from aitop.core.system.memory import SystemMemoryMonitor
from aitop.core.system.cpu import CPUStats
from aitop.ui.display import Display
from aitop.ui.components.header import HeaderComponent
from aitop.ui.components.footer import FooterComponent
from aitop.ui.components.tabs import TabsComponent
from aitop.ui.components.overview import OverviewPanel
from aitop.ui.components.gpu_panel import GPUPanel
from aitop.ui.components.process_panel import ProcessPanel
from aitop.ui.components.memory_panel import MemoryPanel
from aitop.ui.components.cpu_panel import CPUPanel


@dataclass
class SystemData:
    """Container for system monitoring data."""
    gpu_info: List[Tuple[Any, str]]
    gpu_processes: List[Any]
    processes: List[Any]
    memory_stats: Any
    memory_types: Any
    cpu_stats: Any
    primary_vendor: str


class DataCollector(threading.Thread):
    """Background thread for collecting system data."""
    
    def __init__(self, update_interval: float):
        """Initialize the data collector.
        
        Args:
            update_interval: Time between updates in seconds
        """
        super().__init__(daemon=True)  # Daemon thread will exit when main thread exits
        self.update_interval = update_interval
        self.queue = Queue(maxsize=1)  # Only keep latest data
        self.running = True
        
        # Initialize monitors
        self.gpu_monitors = GPUMonitorFactory.create_monitors()
        self.ai_monitor = AIProcessMonitor()
        self.memory_monitor = SystemMemoryMonitor()
        self.vendors = GPUMonitorFactory.detect_vendors()
    
    def run(self) -> None:
        """Run the data collection loop."""
        while self.running:
            try:
                # Get GPU info
                gpu_info = []
                for monitor, vendor in zip(self.gpu_monitors, self.vendors):
                    gpus = monitor.get_gpu_info() if monitor else []
                    gpu_info.extend([(gpu, vendor) for gpu in gpus])
                
                # Extract GPU processes
                gpu_processes = []
                for monitor, _ in zip(self.gpu_monitors, self.vendors):
                    if monitor:
                        gpus = monitor.get_gpu_info()
                        for gpu in gpus:
                            gpu_processes.extend(gpu.processes)
                
                # Update all stats
                processes = self.ai_monitor.get_ai_processes(gpu_processes)
                memory_stats = self.memory_monitor.get_memory_stats()
                memory_types = self.memory_monitor.get_memory_by_type()
                cpu_stats = CPUStats.get_stats()
                primary_vendor = self.vendors[0] if self.vendors else 'none'
                
                # Package data
                data = SystemData(
                    gpu_info=gpu_info,
                    gpu_processes=gpu_processes,
                    processes=processes,
                    memory_stats=memory_stats,
                    memory_types=memory_types,
                    cpu_stats=cpu_stats,
                    primary_vendor=primary_vendor
                )
                
                # Update queue, removing old data if necessary
                if self.queue.full():
                    try:
                        self.queue.get_nowait()
                    except:
                        pass
                self.queue.put(data)
                
                time.sleep(self.update_interval)
            except Exception:
                # Log error but keep running
                time.sleep(self.update_interval)
    
    def stop(self) -> None:
        """Stop the data collection thread."""
        self.running = False


class AITop:
    """Main application class."""
    
    def __init__(self, stdscr):
        """Initialize the application.
        
        Args:
            stdscr: Curses window object
        """
        # Initialize display and UI components
        self.display = Display(stdscr)
        self.header = HeaderComponent(self.display)
        self.footer = FooterComponent(self.display)
        self.tabs = TabsComponent(self.display)
        self.overview = OverviewPanel(self.display)
        self.gpu_panel = GPUPanel(self.display)
        self.process_panel = ProcessPanel(self.display)
        self.memory_panel = MemoryPanel(self.display)
        self.cpu_panel = CPUPanel(self.display)
        
        # Application state
        self.running = True
        self.selected_tab = 0
        self.sort_by = 'cpu_percent'
        self.sort_reverse = True
        self.scroll_position = 0
        self.input_timeout = 50    # milliseconds - balanced input checking
        self.needs_redraw = True   # Track if UI needs updating
        
        # Initialize data collector thread
        self.collector = DataCollector(update_interval=1.0)
        self.collector.start()
        
        # Initialize system data
        self.system_data = None
    
    def handle_input(self) -> None:
        """Handle user input."""
        try:
            key = self.display.stdscr.getch()
            
            if key == curses.ERR:  # No input available
                return
                
            self.needs_redraw = True  # Input usually requires redraw
            
            if key == ord('q'):
                self.running = False
            elif key == ord('c'):
                self.sort_by = 'cpu_percent'
                self.sort_reverse = True
            elif key == ord('m'):
                self.sort_by = 'memory_percent'
                self.sort_reverse = True
            elif key == ord('h'):
                self.sort_reverse = not self.sort_reverse
            elif key in [curses.KEY_LEFT, curses.KEY_RIGHT]:
                self.selected_tab = self.tabs.handle_tab_input(key, self.selected_tab)
                self.scroll_position = 0
            elif key in [curses.KEY_UP, curses.KEY_DOWN] and self.system_data:
                if self.selected_tab == 1:  # AI Processes tab
                    self.scroll_position = self.process_panel.handle_scroll(
                        key, self.scroll_position, self.system_data.processes
                    )
        except curses.error:
            pass
    
    def render(self) -> None:
        """Render the complete interface."""
        try:
            # Check for resize first to avoid unnecessary renders
            height, width = self.display.get_dimensions()
            if (height, width) != (self.display.height, self.display.width):
                self.display.handle_resize()
                self.needs_redraw = True
            
            # Get latest data if available
            try:
                new_data = self.collector.queue.get_nowait()
                self.system_data = new_data
                self.needs_redraw = True
            except:
                pass
            
            # Only redraw if needed and we have data
            if self.needs_redraw and self.system_data:
                self.display.clear()
                
                # Render common elements
                self.header.render(self.system_data.primary_vendor)
                self.tabs.render(self.selected_tab, 1)
                
                # Render tab-specific content
                if self.selected_tab == 0:  # Overview
                    self.overview.render(
                        self.system_data.gpu_info,
                        self.system_data.processes,
                        self.system_data.memory_stats,
                        self.system_data.cpu_stats,
                        self.system_data.primary_vendor
                    )
                elif self.selected_tab == 1:  # AI Processes
                    self.process_panel.render(
                        self.system_data.processes,
                        [gpu for gpu, _ in self.system_data.gpu_info],
                        3, 2,
                        self.sort_by,
                        self.sort_reverse,
                        self.scroll_position
                    )
                elif self.selected_tab == 2:  # GPU
                    self.gpu_panel.render(self.system_data.gpu_info)
                elif self.selected_tab == 3:  # Memory
                    self.memory_panel.render(
                        self.system_data.memory_stats,
                        self.system_data.memory_types
                    )
                elif self.selected_tab == 4:  # CPU
                    self.cpu_panel.render(self.system_data.cpu_stats)
                    
                self.footer.render()
                self.display.refresh()
                self.needs_redraw = False
            
        except curses.error:
            pass
    
    def cleanup(self) -> None:
        """Clean up resources."""
        if self.collector:
            self.collector.stop()
            self.collector.join(timeout=1.0)
    
    def run(self) -> None:
        """Main application loop."""
        try:
            while self.running:
                self.handle_input()
                self.render()
        finally:
            self.cleanup()


def handle_resize(signo, frame):
    """Handle terminal resize event."""
    curses.endwin()  # End current window
    curses.initscr()  # Reinitialize screen
    
def _main(stdscr) -> int:
    """Initialize and run the application.
    
    Args:
        stdscr: Curses window object
        
    Returns:
        Exit code
    """
    # Handle terminal resize
    signal.signal(signal.SIGWINCH, handle_resize)
    
    try:
        app = AITop(stdscr)
        app.run()
    except Exception as e:
        # Restore terminal
        curses.nocbreak()
        stdscr.keypad(False)
        curses.echo()
        curses.endwin()
        
        print(f"Error: {str(e)}", file=sys.stderr)
        return 1
        
    return 0


def main():
    """Entry point for the application."""
    try:
        return curses.wrapper(_main)
    except KeyboardInterrupt:
        return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
"""System memory monitoring functionality."""

from dataclasses import dataclass
from typing import Dict, Any
import psutil


@dataclass
class MemoryStats:
    """Container for memory statistics."""
    total: float  # Total physical memory in bytes
    used: float   # Used memory in bytes
    free: float   # Free memory in bytes
    percent: float  # Memory usage percentage
    swap_total: float  # Total swap memory in bytes
    swap_used: float   # Used swap memory in bytes
    swap_free: float   # Free swap memory in bytes
    swap_percent: float  # Swap usage percentage


class SystemMemoryMonitor:
    """Monitors system memory usage."""
    
    @staticmethod
    def get_memory_stats() -> MemoryStats:
        """Get current memory statistics."""
        vm = psutil.virtual_memory()
        swap = psutil.swap_memory()
        
        return MemoryStats(
            total=vm.total,
            used=vm.used,
            free=vm.free,
            percent=vm.percent,
            swap_total=swap.total,
            swap_used=swap.used,
            swap_free=swap.free,
            swap_percent=swap.percent
        )
    
    @staticmethod
    def get_memory_by_type() -> Dict[str, float]:
        """Get memory usage broken down by type (in bytes)."""
        vm = psutil.virtual_memory()
        return {
            'available': vm.available,
            'buffers': getattr(vm, 'buffers', 0),
            'cached': getattr(vm, 'cached', 0),
            'shared': getattr(vm, 'shared', 0),
            'slab': getattr(vm, 'slab', 0)
        }
    
    @staticmethod
    def bytes_to_gb(bytes_value: float) -> float:
        """Convert bytes to gigabytes."""
        return bytes_value / (1024 ** 3)
    
    def get_formatted_stats(self) -> Dict[str, Any]:
        """Get memory statistics formatted in GB with percentages."""
        stats = self.get_memory_stats()
        memory_types = self.get_memory_by_type()
        
        return {
            'ram': {
                'total_gb': self.bytes_to_gb(stats.total),
                'used_gb': self.bytes_to_gb(stats.used),
                'free_gb': self.bytes_to_gb(stats.free),
                'percent': stats.percent,
                'details': {
                    name: self.bytes_to_gb(value)
                    for name, value in memory_types.items()
                }
            },
            'swap': {
                'total_gb': self.bytes_to_gb(stats.swap_total),
                'used_gb': self.bytes_to_gb(stats.swap_used),
                'free_gb': self.bytes_to_gb(stats.swap_free),
                'percent': stats.swap_percent
            }
        }


#!/usr/bin/env python3
"""NVIDIA-specific GPU monitoring implementation."""

import os
import subprocess
from pathlib import Path
from typing import List, Optional

from .base import BaseGPUMonitor, GPUInfo


class NvidiaGPUMonitor(BaseGPUMonitor):
    """NVIDIA GPU monitoring implementation."""
    
    def _find_smi(self) -> Optional[Path]:
        """Find nvidia-smi executable."""
        try:
            # Try running nvidia-smi directly first
            subprocess.run(['nvidia-smi', '--version'],
                         capture_output=True, text=True, check=True)
            # Use the known working path
            return Path('/usr/bin/nvidia-smi')
            
        except (subprocess.CalledProcessError, FileNotFoundError):
            # Fallback to common locations
            common_paths = [
                '/usr/bin/nvidia-smi',
                '/usr/local/bin/nvidia-smi'
            ]
            
            for path in common_paths:
                p = Path(path)
                if p.exists():
                    try:
                        subprocess.run([str(p), '--version'],
                                    capture_output=True, text=True, check=True)
                        return p
                    except (subprocess.CalledProcessError, Exception):
                        continue
                        
        return None

    def get_gpu_info(self) -> List[GPUInfo]:
        """Get NVIDIA GPU information using nvidia-smi."""
        if not self.smi_path:
            return []

        try:
            # Get GPU information
            cmd = [
                str(self.smi_path),
                '--query-gpu=index,name,utilization.gpu,memory.used,memory.total,temperature.gpu,power.draw,power.limit,compute_mode',
                '--format=csv,noheader,nounits'
            ]
            result = self._run_smi_command(cmd)
            if not result:
                return []
            
            # Get process information
            cmd_proc = [
                str(self.smi_path),
                '--query-compute-apps=gpu_uuid,pid,used_memory,process_name',
                '--format=csv,nounits,noheader'
            ]
            try:
                proc_result = self._run_smi_command(cmd_proc)
                if not proc_result or not proc_result.strip():
                    proc_result = ""
            except Exception:
                proc_result = ""
            
            gpus = []
            for line in result.strip().split('\n'):
                if not line.strip():
                    continue
                    
                values = line.split(', ')
                if len(values) >= 8:
                    index = int(values[0])
                    
                    # Parse processes
                    processes = []
                    for proc_line in proc_result.strip().split('\n'):
                        if proc_line.strip():
                            try:
                                proc_values = proc_line.split(', ')
                                if len(proc_values) >= 4:
                                    processes.append({
                                        'pid': int(proc_values[1]),
                                        'memory': float(proc_values[2]),
                                        'name': proc_values[3].strip()
                                    })
                            except (ValueError, IndexError):
                                continue
                    
                    try:
                        power_draw = float(values[6]) if values[6] != '[N/A]' else 0
                        power_limit = float(values[7]) if values[7] != '[N/A]' else 0
                        
                        gpu_info = GPUInfo(
                            index=index,
                            name=values[1].strip(),
                            utilization=float(values[2]),
                            memory_used=float(values[3]),
                            memory_total=float(values[4]),
                            temperature=float(values[5]),
                            power_draw=power_draw,
                            power_limit=power_limit,
                            processes=processes
                        )
                        gpus.append(gpu_info)
                    except ValueError:
                        continue
            
            return gpus
        except (ValueError, IndexError):
            return []



#!/usr/bin/env python3
"""Base display handling functionality."""

import curses
from typing import Tuple, Optional, Dict, Any


class Display:
    """Handles the terminal UI display."""
    
    def __init__(self, stdscr):
        """Initialize the display.
        
        Args:
            stdscr: curses window object
        """
        self.stdscr = stdscr
        self.setup_colors()
        self.height, self.width = stdscr.getmaxyx()
        
        # Initialize curses settings
        self.stdscr.keypad(1)  # Enable keypad
        curses.halfdelay(1)    # Set input mode to avoid blocking
        curses.use_env(True)   # Enable terminal size detection
        
        # Create a pad for double buffering
        self.pad = curses.newpad(self.height + 1, self.width + 1)
        self.pad.keypad(1)
        
        # Try to hide cursor - some terminals may not support this
        try:
            curses.curs_set(0)
        except curses.error:
            # If cursor hiding fails, don't treat it as fatal
            pass
    
    def setup_colors(self) -> None:
        """Initialize color pairs safely."""
        try:
            curses.start_color()
            curses.use_default_colors()
            
            # Define color pairs
            pairs = [
                (1, curses.COLOR_GREEN, -1),     # Normal
                (2, curses.COLOR_BLACK, curses.COLOR_GREEN),  # Selected
                (3, curses.COLOR_YELLOW, -1),    # Warning
                (4, curses.COLOR_RED, -1),       # Critical
                (5, curses.COLOR_CYAN, -1),      # Info
                (6, curses.COLOR_MAGENTA, -1),   # AI Process
                (7, curses.COLOR_BLUE, -1),      # Header
                (8, curses.COLOR_WHITE, -1),     # Footer
            ]
            
            for pair_id, fg, bg in pairs:
                curses.init_pair(pair_id, fg, bg)
        except Exception:
            pass
    
    def get_dimensions(self) -> Tuple[int, int]:
        """Get current terminal dimensions."""
        self.height, self.width = self.stdscr.getmaxyx()
        return self.height, self.width
    
    def safe_addstr(self, y: int, x: int, text: str, attr: Optional[int] = None) -> None:
        """Safely add a string to the screen.
        
        Args:
            y: Y coordinate
            x: X coordinate
            text: Text to display
            attr: Curses attribute (color pair, etc.)
        """
        try:
            if y >= self.height or x >= self.width:
                return
                
            max_length = self.width - x
            if len(text) > max_length:
                text = text[:max_length]
                
            if attr is not None:
                self.pad.addstr(y, x, text, attr)
            else:
                self.pad.addstr(y, x, text)
        except curses.error:
            pass
    
    def get_color(self, value: float) -> int:
        """Get appropriate color based on value percentage.
        
        Args:
            value: Percentage value (0-100)
            
        Returns:
            Curses color pair
        """
        try:
            if value >= 80:
                return curses.color_pair(4)  # Red
            elif value >= 60:
                return curses.color_pair(3)  # Yellow
            return curses.color_pair(1)      # Green
        except Exception:
            return curses.A_NORMAL
    
    def create_bar(self, value: float, width: int) -> Tuple[str, int]:
        """Create a progress bar.
        
        Args:
            value: Percentage value (0-100)
            width: Width of the bar in characters
            
        Returns:
            Tuple of (bar_string, color_attr)
        """
        filled_width = int(value * width / 100)
        bar = '█' * filled_width + '░' * (width - filled_width)
        color = self.get_color(value)
        
        return bar, color
    
    def clear(self) -> None:
        """Clear the screen."""
        self.pad.clear()
    
    def refresh(self) -> None:
        """Refresh the screen using double buffering."""
        try:
            # Copy pad content to screen
            self.pad.noutrefresh(0, 0, 0, 0, self.height - 1, self.width - 1)
            curses.doupdate()
        except curses.error:
            pass
    
    def handle_resize(self) -> None:
        """Handle terminal resize event."""
        # Get new dimensions
        new_height, new_width = self.stdscr.getmaxyx()
        
        # Only process if dimensions actually changed
        if new_height != self.height or new_width != self.width:
            self.height = new_height
            self.width = new_width
            
            # Recreate pad with new dimensions
            self.pad = curses.newpad(self.height + 1, self.width + 1)
            self.pad.keypad(1)
            
            # Clear and resize
            curses.resize_term(self.height, self.width)
            curses.flushinp()  # Flush input buffer
            
            # Reset terminal state
            self.stdscr.clear()
            self.stdscr.refresh()
            
            # Force immediate refresh of pad
            self.refresh()
    
    def create_window(self, height: int, width: int, y: int, x: int) -> Any:
        """Create a new window.
        
        Args:
            height: Window height
            width: Window width
            y: Y coordinate
            x: X coordinate
            
        Returns:
            Curses window object
        """
        try:
            return curses.newwin(height, width, y, x)
        except curses.error:
            return None
    
    def center_text(self, text: str, width: Optional[int] = None) -> str:
        """Center text within given width.
        
        Args:
            text: Text to center
            width: Width to center within (defaults to screen width)
            
        Returns:
            Centered text string
        """
        if width is None:
            width = self.width
        return text.center(width)
    
    def truncate_text(self, text: str, max_length: int, ellipsis: str = '...') -> str:
        """Truncate text to specified length.
        
        Args:
            text: Text to truncate
            max_length: Maximum length
            ellipsis: String to indicate truncation
            
        Returns:
            Truncated text
        """
        if len(text) <= max_length:
            return text
        return text[:max_length-len(ellipsis)] + ellipsis
    
    def render_table(self, headers: Dict[str, int], rows: list, start_y: int, 
                    start_x: int, max_rows: Optional[int] = None) -> int:
        """Render a table with given headers and rows.
        
        Args:
            headers: Dict of column name to width
            rows: List of row data
            start_y: Starting Y coordinate
            start_x: Starting X coordinate
            max_rows: Maximum number of rows to display
            
        Returns:
            Number of rows rendered
        """
        y = start_y
        
        # Render headers
        x = start_x
        for header, width in headers.items():
            self.safe_addstr(y, x, header.ljust(width)[:width], 
                           curses.color_pair(5) | curses.A_BOLD)
            x += width + 1
        y += 1
        
        # Render rows
        if max_rows is None:
            max_rows = self.height - y - 1
            
        for row in rows[:max_rows]:
            x = start_x
            for col, width in headers.items():
                value = str(row.get(col, ''))
                self.safe_addstr(y, x, value.ljust(width)[:width])
                x += width + 1
            y += 1
            
        return min(len(rows), max_rows)
